using System;
using Celeste.Mod.Entities;
using System.Linq;
using System.Text;
using Celeste;
using Monocle;
using On.Celeste;
using static Celeste.Mod.APMmeter.Display;
using System.Collections.Generic;
using MonoMod.Utils;

namespace Celeste.Mod.APMmeter;

public class APMmeterModuleSettings : EverestModuleSettings
{

    [SettingName("enabled")]
    public bool Enabled { get; set; } = true;

    [SettingName("calcMethod")]
    [SettingSubText("calcMethodSubtext")]
    public CalcMethod CalcMethod { get; set; } = CalcMethod.GapSaving;
    public AccountingInputsSubMenu InputAccounting { get; set; } = new AccountingInputsSubMenu();
    [SettingSubMenu]
    public class AccountingInputsSubMenu
    {
        [SettingName("Dash")]
        public Options Dash { get; set; } = Options.OnlyPresses;

        [SettingName("Demo")]
        public Options DemoDash { get; set; } = Options.OnlyPresses;

        [SettingName("Jump")]
        public Options Jump { get; set; } = Options.OnlyPresses;

        [SettingName("Dirs")]
        public Options Directions { get; set; } = Options.OnlyPresses;

        [SettingName("GrabPress")]
        [SettingSubText("GrabPressSubtext")]
        public GrabOptions GrabPress { get; set; } = GrabOptions.All;

        [SettingName("GrabRelease")]
        [SettingSubText("GrabReleaseSubtext")]
        public GrabOptions GrabRelease { get; set; } = GrabOptions.All;
    }

    private float BufferSizeRaw = 2.5f;
    [SettingName("bufferSize")]
    [SettingSubText("bufferSizeSubtext")]
    [SettingInGame(false)]
    [SettingNumberInput(false, 5)]
    public float BufferSize
    {
        get => BufferSizeRaw;
        set => BufferSizeRaw = Calc.Clamp(value, 0.1f, 60f);
    }

    private float RefreshIntervalRaw = 0.3f;
    [SettingName("refreshInterval")]
    [SettingSubText("refreshIntervalSubtext")]
    [SettingInGame(false)]
    [SettingNumberInput(false)]
    public float DisplayInterval
    {
        get => RefreshIntervalRaw;
        set => RefreshIntervalRaw = Calc.Clamp(value, 0.0167f, 60f);
    }

    [SettingName("countPerMinute")]
    [SettingSubText("countPerMinuteSubtext")]
    public bool CountPerMinute { get; set; } = true;

    [SettingName("decimalPartLength")]
    [SettingRange(0, 5)]
    public int DecimalPartLength { get; set; } = 2;

    public TextParametersSubMenu TextCustomization { get; set; } = new TextParametersSubMenu();
    [SettingSubMenu]
    public class TextParametersSubMenu
    {
        [SettingName("posX")]
        [SettingRange(-10, 330, true)]
        public int PosX { get; set; } = 2;

        [SettingName("posY")]
        [SettingRange(-10, 194, true)]
        public int PosY { get; set; } = 155;

        [SettingName("alignment")]
        public Alignment Alignment { get; set; } = Alignment.Left;

        [SettingName("size")]
        [SettingSubText("SizeSubtext")]
        [SettingRange(1, 500, true)]
        public int Size { get; set; } = 100;

        [SettingName("useColorSetting")]
        public UseColorSetting UseColorSetting { get; set; } = UseColorSetting.RGB;

        [SettingName("r")]
        [SettingRange(0, 255, true)]
        public int Red { get; set; } = 255;

        [SettingName("g")]
        [SettingRange(0, 255, true)]
        public int Green { get; set; } = 255;

        [SettingName("b")]
        [SettingRange(0, 255, true)]
        public int Blue { get; set; } = 255;

        [SettingName("hexColor")]
        [SettingMinLength(6)]
        [SettingMaxLength(6)]
        public string HexColor { get; set; } = "FFFFFF";

        [SettingName("outline")]
        [SettingSubText("OutlineSubtext")]
        public bool Outline { get; set; } = true;

        [SettingName("opacity")]
        [SettingRange(0, 100, true)]
        public int Opacity { get; set; } = 100;


        [SettingName("showCounterTitleMod")]
        public ShowCounterTitleMod ShowCounterTitle { get; set; } = ShowCounterTitleMod.None;

        [SettingName("customCounterTitle")]
        [SettingSubText("customCounterTitleSubtext")]
        [SettingMinLength(0)]
        [SettingMaxLength(100)]
        public string CustomCounterTitle { get; set; } = "";
    }

    [SettingName("showBG")]
    [SettingSubText("ShowBGSubtext")]
    public bool ShowBG { get; set; } = true;
    public BGSettings BackGroundSettings { get; set; } = new BGSettings();
    [SettingSubMenu]

    public class BGSettings
    {
        private float PeakLifeTimeRaw = 2f;
        [SettingName("peakLifetime")]
        [SettingSubText("peakLifetimeSubtext")]
        [SettingInGame(false)]
        [SettingNumberInput(false, 5)]
        public float PeakLifeTime
        {
            get => PeakLifeTimeRaw;
            set => PeakLifeTimeRaw = Calc.Clamp(value, 0f, 10f);
        }

        [SettingName("peakDecreaseSpeed")]
        [SettingSubText("peakDecreaseSpeedSubtext")]
        [SettingRange(1, 100, true)]
        public int PeakDecreaseSpeed { get; set; } = 20;
    }


    [SettingName("showResultRoomCPM")]
    [SettingSubText("showResultRoomCPMSubtext")]
    public bool ShowResultRoomCPM { get; set; } = true;
    public ResultRoomCPM ResultRoomValueSettings { get; set; } = new ResultRoomCPM();
    [SettingSubMenu]

    public class ResultRoomCPM
    {
        [SettingName("resultSize")]
        [SettingSubText("resultSizeSubtext")]
        [SettingRange(1, 500, true)]
        public int ResultSize { get; set; } = 140;


        private float DurationRaw = 4f;
        [SettingName("duration")]
        [SettingSubText("durationSubtext")]
        [SettingInGame(false)]
        [SettingNumberInput(false, 5)]
        public float Duration
        {
            get => DurationRaw;
            set => DurationRaw = Calc.Clamp(value, 1f, 10f);
        }

        [SettingName("copyTextColor")]
        public bool CopyTextParams { get; set; } = false;
    }
}