﻿using System;
using Celeste.Mod.Entities;
using System.Linq;
using System.Text;
using Celeste;
using Microsoft.Xna.Framework;
using Monocle;
using Microsoft.Xna.Framework.Graphics;
using AsmResolver.DotNet.Code.Cil;
using On.Celeste;
using FMOD;
using static System.Runtime.InteropServices.JavaScript.JSType;
using Microsoft.Xna.Framework.Input;
using On.MonoMod;
using System.Runtime.CompilerServices;
using IL.Celeste;
using AsmResolver.DotNet;
using System.Linq.Expressions;

namespace Celeste.Mod.APMmeter;

public class Display : Entity
{
    public static string dashName = true? "dash" : "dashe";
    
    private static void Log(object input)
    {
        if (Type.GetTypeCode(input.GetType()) != TypeCode.String)
            Logger.Log(LogLevel.Info, "Meter", input.ToString());
        else
        {
            string result = input.ToString();
            Logger.Log(LogLevel.Info, "Meter", result);
        }

    }
    public enum CalcMethod
    {
        GapSaving,
        InputSaving,
        Utug,
        ByRoom
    }
    public enum Options
    {
        All,
        OnlyPresses,
        OnlyReleases,
        None,
    }
    public enum GrabOptions
    {
        All,
        OnlyNormal,
        OnlyHoldable,
        None
    }
    public enum Alignment
    {
        Left,
        Center,
        Right
    }
    public enum ShowCounterTitleMod
    {
        None,
        Before,
        After
    }
    public enum UseColorSetting
    {
        RGB,
        Hex
    }

    //Settings
    public static CalcMethod method => APMmeterModule.Settings.CalcMethod;
    private static Options dash => APMmeterModule.Settings.InputAccounting.Dash;
    private static Options demo => APMmeterModule.Settings.InputAccounting.DemoDash;
    private static Options jump => APMmeterModule.Settings.InputAccounting.Jump;
    private static Options dir => APMmeterModule.Settings.InputAccounting.Directions;

    private static GrabOptions grabPress => APMmeterModule.Settings.InputAccounting.GrabPress;
    private static GrabOptions grabRelease => APMmeterModule.Settings.InputAccounting.GrabRelease;

    private static float bufferSize => APMmeterModule.Settings.BufferSize;
    private static float displayInterval => APMmeterModule.Settings.DisplayInterval;
    private static bool countPerMinute => APMmeterModule.Settings.CountPerMinute;
    private static int decimalPartLength => APMmeterModule.Settings.DecimalPartLength;
    private static Alignment align => APMmeterModule.Settings.TextCustomization.Alignment;

    private static int red => APMmeterModule.Settings.TextCustomization.UseColorSetting == UseColorSetting.RGB ?
        APMmeterModule.Settings.TextCustomization.Red :
        HexToColor(APMmeterModule.Settings.TextCustomization.HexColor, 0);
    private static int green => APMmeterModule.Settings.TextCustomization.UseColorSetting == UseColorSetting.RGB ?
        APMmeterModule.Settings.TextCustomization.Green :
        HexToColor(APMmeterModule.Settings.TextCustomization.HexColor, 2);
    private static int blue => APMmeterModule.Settings.TextCustomization.UseColorSetting == UseColorSetting.RGB ?
        APMmeterModule.Settings.TextCustomization.Blue :
        HexToColor(APMmeterModule.Settings.TextCustomization.HexColor, 4);

    private static int opacity => APMmeterModule.Settings.TextCustomization.Opacity;
    private static int size => APMmeterModule.Settings.TextCustomization.Size;
    private static float grayScale => (float)(0.299f * red) + (float)(0.587f * green) + (float)(0.115f * blue);
    //Input counters
    private struct Gap
    {
        public float lifeTime;
        public float size;
        public Gap(float lifeTime, float size)
        {
            this.lifeTime = lifeTime;
            this.size = size;
        }
    }
    private static bool grabPressed;

    public static int decentInputs;
    private static int secondaryInputs;
    public static float displayAmount;

    private static Vector2 previousAim;

    private static float displayIntervalTimer;

    //Display

    private static string resultText = "0";
    private static Vector2 textJustify;
    private static ShowCounterTitleMod showTitle => APMmeterModule.Settings.TextCustomization.ShowCounterTitle;
    private static MTexture background;
    private static bool showBG => APMmeterModule.Settings.ShowBG;
    private static Vector2 TextPos => new Vector2(APMmeterModule.Settings.TextCustomization.PosX, APMmeterModule.Settings.TextCustomization.PosY);
    private static Vector2 offset => size / 100f * new Vector2(0f, 5.9f);
    private static bool outline => APMmeterModule.Settings.TextCustomization.Outline;

    private static Gap peak;

    //Srrc
    public static bool showResultRoomCPM => APMmeterModule.Settings.ShowResultRoomCPM;
    public static float srrcAmount;
    private static float srrcDuration => APMmeterModule.Settings.ResultRoomValueSettings.Duration;
    public struct ResultRoomCPM
    {
        public float timer;
        public Vector2 pos;
        public float number;

        public ResultRoomCPM(float number)
        {
            timer = 0f;
            pos = TextPos;
            this.number = number;
        }
    }
    private static SimpleCurve srrcPosCurve => new SimpleCurve(TextPos,
            new Vector2(TextPos.X, TextPos.Y - 14f * (size / 100f) * (APMmeterModule.Settings.ResultRoomValueSettings.ResultSize / 140f)),
            new Vector2(TextPos.X, TextPos.Y - 10f * (size / 100f)));
    private static SimpleCurve srrcOpacityCurve = new SimpleCurve(Vector2.Zero, new Vector2(2f, 0f), new Vector2(1.3f, 2f));
    private static SimpleCurve srrcSizeCurve = new SimpleCurve(new Vector2(0f, 1f), new Vector2(2f, APMmeterModule.Settings.ResultRoomValueSettings.ResultSize / 100f), new Vector2(1f, ((APMmeterModule.Settings.ResultRoomValueSettings.ResultSize / 50f) + 1f) / 3f));
    private static ResultRoomCPM[] resultCPM;


    //Input saving method
    private static float gapTimer;
    private static Gap[] gaps;
    private static float gapSumSize;

    //Input saving method
    private static float[] inputs;

    //byRoom method
    private static bool firstTime;
    private static Vector2? prevRespawnPoint;
    public static float CRtimer;


    public static int HexToColor(string input, int num)
    {
        int result = 0;
        for(int c=num; c<num+2; c++)
        {
            if (input[c] == '1')
                result += (int)MathF.Pow(16, 1 - (c - num));
            else if (input[c] == '2')
                result += 2*(int)MathF.Pow(16, 1 - (c - num));
            else if (input[c] == '3')
                result += 3*(int)MathF.Pow(16, 1 - (c - num));
            else if (input[c] == '4')
                result += 4*(int)MathF.Pow(16, 1 - (c - num));
            else if (input[c] == '5')
                result += 5*(int)MathF.Pow(16, 1 - (c - num));
            else if (input[c] == '6')
                result += 6*(int)MathF.Pow(16, 1 - (c - num));
            else if (input[c] == '7')
                result += 7*(int)MathF.Pow(16, 1 - (c - num));
            else if (input[c] == '8')
                result += 8*(int)MathF.Pow(16, 1 - (c - num));
            else if (input[c] == '9')
                result += 9*(int)MathF.Pow(16, 1 - (c - num));
            else if (input[c] == 'A' || input[c] == 'a')
                result += 10*(int)MathF.Pow(16, 1 - (c - num));
            else if (input[c] == 'B' || input[c] == 'b')
                result += 11*(int)MathF.Pow(16, 1 - (c - num));
            else if (input[c] == 'C' || input[c] == 'c')
                result += 12*(int)MathF.Pow(16, 1 - (c - num));
            else if (input[c] == 'D' || input[c] == 'd')
                result += 13*(int)MathF.Pow(16, 1 - (c - num));
            else if (input[c] == 'E' || input[c] == 'e')
                result += 14*(int)MathF.Pow(16, 1 - (c - num));
            else if (input[c] == 'F' || input[c] == 'f')
                result += 15*(int)MathF.Pow(16, 1 - (c - num));
        }
        return result;
    }

    private void NilVariables()
    {
        decentInputs = 0;
        secondaryInputs = 0;
        CRtimer = 0f;
        displayIntervalTimer = 0f;
        gaps = new Gap[0];
        inputs = new float[0];
        gapSumSize = 0f;
        grabPressed = false;
        firstTime = true;
        resultCPM = new ResultRoomCPM[0];
        peak = new Gap(0f,0f);
    }
    public Display()
    {

        background = GFX.Gui["IPMmetertriangleBG"];
        AddTag(Tags.HUD);
        AddTag(Tags.Global);
        AddTag(Tags.TransitionUpdate);  
        AddTag(Tags.FrozenUpdate);
        AddTag(Tags.Persistent);
        AddTag(Tags.PauseUpdate);

        Visible = true;

        NilVariables();
    }
    public override void Update()
    {
        base.Update();
        if (!APMmeterModule.Settings.Enabled)
            return;

        Player player = Scene.Tracker.GetEntity<Player>();

        if (grabPressed && player != null)
        {
            if(player.Holding!=null)
            {
                if (grabPress == GrabOptions.OnlyHoldable)
                    decentInputs++;
                else if(grabPress == GrabOptions.OnlyNormal && decentInputs > 0)
                    decentInputs--;

                grabPressed = false;
            }
        }

        if (Input.Dash.Binding.Pressed(Input.Dash.GamepadIndex, Input.Dash.Threshold) &&
            (dash == Options.All || dash == Options.OnlyPresses))
            decentInputs++;
        if (Input.Dash.Binding.Released(Input.Dash.GamepadIndex, Input.Dash.Threshold) &&
            (dash == Options.All || dash == Options.OnlyReleases))
            decentInputs++;

        if (Input.CrouchDash.Binding.Pressed(Input.CrouchDash.GamepadIndex, Input.CrouchDash.Threshold) &&
            (demo == Options.All || demo == Options.OnlyPresses))
            decentInputs++;
        if (Input.CrouchDash.Binding.Released(Input.CrouchDash.GamepadIndex, Input.CrouchDash.Threshold) &&
            (demo == Options.All || demo == Options.OnlyReleases))
            decentInputs++;

        if (Input.Jump.Binding.Pressed(Input.Jump.GamepadIndex, Input.Jump.Threshold) &&
            (jump == Options.All || jump == Options.OnlyPresses))
            decentInputs++;
        if (Input.Jump.Binding.Released(Input.Jump.GamepadIndex, Input.Jump.Threshold) &&
            (jump == Options.All || jump == Options.OnlyReleases))
            decentInputs++;


        if (Input.MoveX != 0f && previousAim.X == 0f &&
            (dir == Options.All || dir == Options.OnlyPresses))
            decentInputs++;

        if (Input.MoveX == 0f && previousAim.X != 0f &&
            (dir == Options.All || dir == Options.OnlyReleases))
            decentInputs++;

        if (Input.MoveY != 0f && previousAim.Y == 0f &&
            (dir == Options.All || dir == Options.OnlyPresses))
            decentInputs++;

        if (Input.MoveY == 0f && previousAim.Y != 0f &&
            (dir == Options.All || dir == Options.OnlyReleases))
            decentInputs++;

        previousAim = new Vector2(Input.MoveX, Input.MoveY);

        if (Input.Grab.Pressed && grabPress != GrabOptions.None)
        {
            if(grabPress != GrabOptions.All)
            {
                grabPressed = true;
                if (grabPress == GrabOptions.OnlyNormal)
                    decentInputs++;
            }
            else
                decentInputs++;

        }
        if (Input.Grab.Released && grabRelease != GrabOptions.None)
        {
            grabPressed = false;

            switch (grabRelease)
            {
                case GrabOptions.OnlyNormal:
                {
                    if (player != null && player.Holding == null)
                        decentInputs++;
                    break;
                }
                case GrabOptions.OnlyHoldable:
                {
                    if (player != null && player.Holding != null)
                        decentInputs++;
                    break;
                }
                case GrabOptions.All:
                {
                    decentInputs++;
                    break;
                }
            }
        }
    }
    public void RenderUpdate() //сделать настройки юзабельными, не забыть потестить
    {
        //вычисление кпм/кпс

        if (method != CalcMethod.ByRoom && showResultRoomCPM)
        {
            srrcAmount += decentInputs;
            CRtimer += Engine.DeltaTime;

            if (Scene.Tracker.GetEntity<Player>() != null)
            {
                Player player = Scene.Tracker.GetEntity<Player>();
                if(player.JustRespawned)
                {
                    CRtimer = 0f;
                    srrcAmount = 0f;
                }

                if (player.level.Session.RespawnPoint != prevRespawnPoint)
                {
                    RespawnChanged((float)(srrcAmount / CRtimer));

                    CRtimer = 0f;
                    srrcAmount = 0f;
                }

                prevRespawnPoint = player.level.Session.RespawnPoint;
            }
        }

        switch (method)
        {
            case CalcMethod.GapSaving:
            {
                if (!firstTime)
                    firstTime = true;

                displayIntervalTimer += Engine.DeltaTime;

                if (gapTimer < bufferSize)
                    gapTimer += Engine.DeltaTime;

                if (Scene.Tracker.GetEntity<Player>() != null)
                {
                    Player player = Scene.Tracker.GetEntity<Player>();
                    if (player.JustRespawned)
                    {
                        gapTimer = 0f;
                        decentInputs = 0;
                    }
                }

                if (decentInputs > 0)
                {
                    if (gapTimer!=0)
                    {
                        secondaryInputs = decentInputs;
                        for(decentInputs=secondaryInputs; decentInputs > 0; decentInputs--)
                        {
                            Gap[] bufferArr = new Gap[gaps.Length + 1];

                            if (gaps.Length > 0)
                                for (int c1 = 0; c1 < gaps.Length; c1++)
                                    bufferArr[c1] = gaps[c1];

                            bufferArr[gaps.Length] = (decentInputs==secondaryInputs)?
                                    new Gap(bufferSize, gapTimer) : new Gap(bufferSize, 0.0166667f);
                            gaps = new Gap[bufferArr.Length];
                            gaps = bufferArr;
                        }
                    }
                    decentInputs = 0;
                    gapTimer = 0;
                }

                if (gaps.Length > 0)
                {
                    for (int c = 0; c < gaps.Length; c++)
                    {
                        gaps[c].lifeTime -= Engine.DeltaTime;
                        if (gaps[c].lifeTime < 0f)
                        {
                            if (c == 0 && gaps.Length == 1)
                            {
                                gaps = new Gap[0];
                            }
                            else
                            {
                                Gap[] bufferArr = new Gap[gaps.Length - 1];
                                bufferArr = gaps.Skip(c).ToArray();
                                gaps = new Gap[bufferArr.Length];
                                gaps = bufferArr;
                            }
                        }
                    }
                }
                if(displayIntervalTimer >= displayInterval)
                {
                    if (gaps.Length > 0)
                    {
                        float devider = countPerMinute ? 60.0f : 1.0f;
                        gapSumSize = (float)gaps.Sum(selection => selection.size) + gapTimer;
                        displayAmount = (float)devider / (float)(gapSumSize / gaps.Length);
                    }
                    else
                        displayAmount = 0f;
                    displayIntervalTimer = 0f;
                }
                break;
            }

            case CalcMethod.InputSaving:
            {
                if (!firstTime)
                    firstTime = true;

                displayIntervalTimer += Engine.DeltaTime;

                if (decentInputs > 0)
                {
                    float[] bufferArr = new float[inputs.Length + 1];

                    if (inputs.Length > 0)
                        for (int c = 0; c < inputs.Length; c++)
                            bufferArr[c] = inputs[c];

                    bufferArr[inputs.Length] = bufferSize;
                    inputs = new float[bufferArr.Length];
                    inputs = bufferArr;

                    decentInputs = 0;
                }

                if (inputs.Length > 0)
                {
                    for (int c = 0; c < inputs.Length; c++)
                    {
                        inputs[c] -= Engine.DeltaTime;
                        if (inputs[c] <= 0f)
                        {
                            if (c == 0 && inputs.Length == 1)
                            {
                                inputs = new float[0];
                            }
                            else
                            {
                                float[] bufferArr = new float[inputs.Length - 1];
                                bufferArr = inputs.Skip(c).ToArray();
                                inputs = new float[bufferArr.Length];
                                inputs = bufferArr;
                            }
                        }
                    }
                }

                if (displayIntervalTimer >= displayInterval)
                {
                    if (inputs.Length > 0)
                    {
                        displayAmount = inputs.Length / bufferSize;
                    }
                    else
                        displayAmount = 0f;

                    displayIntervalTimer = 0f;
                }
                break;
            }

            case CalcMethod.Utug:
            {
                if(!firstTime)
                    firstTime = true;

                secondaryInputs += decentInputs;
                decentInputs = 0;

                displayIntervalTimer += Engine.DeltaTime;

                if (displayIntervalTimer >= displayInterval)
                {
                    displayAmount = secondaryInputs / displayInterval;
                    secondaryInputs = 0;

                    displayIntervalTimer = 0f;
                }
                break;
            }
            
            case CalcMethod.ByRoom:
            {
                displayIntervalTimer += Engine.DeltaTime;
                
                if (firstTime)
                {
                    firstTime = false;
                    decentInputs = 0;
                    CRtimer = 0f;
                    displayAmount = 0f;
                }

                CRtimer += Engine.DeltaTime;
                if (Scene.Tracker.GetEntity<Player>() != null)
                {
                    Player player = Scene.Tracker.GetEntity<Player>();
                    if (player.JustRespawned)
                    {
                        CRtimer = 0f;
                        decentInputs = 0;
                        displayAmount = 0f;
                    }

                    if (player.level.Session.RespawnPoint != prevRespawnPoint)
                    {
                        displayAmount = decentInputs / CRtimer;
                        RespawnChanged(displayAmount);
                        CRtimer = 0f;
                        decentInputs = 0;
                    }
                    prevRespawnPoint = player.level.Session.RespawnPoint;
                }

                if (displayIntervalTimer >= displayInterval)
                {
                    displayAmount = (float)decentInputs / CRtimer;
                    displayIntervalTimer = 0f;
                }
                break;
            }
        }
    
    }
    public static void RespawnChanged(float resultRoomCPM)
    {
        ResultRoomCPM[] bufferArr = new ResultRoomCPM[resultCPM.Length + 1];
        if(resultCPM.Length > 0)
        {
            for(int c=0; c<resultCPM.Length; c++)
            {
                bufferArr[c].number = resultCPM[c].number;
                bufferArr[c].timer = resultCPM[c].timer;
                bufferArr[c].pos = resultCPM[c].pos;
            }
        }
        bufferArr[resultCPM.Length] = new ResultRoomCPM(resultRoomCPM);
        resultCPM = new ResultRoomCPM[bufferArr.Length];
        resultCPM = bufferArr;
    }

    public override void Render()
    {
        if (!APMmeterModule.Settings.Enabled)
            return;

        RenderUpdate();

        if (countPerMinute && method!=CalcMethod.GapSaving)
            displayAmount *= 60f;

        if(showBG)
            DrawTextBG();

        textJustify = Vector2.Zero;

        switch (align)
        {
            case Alignment.Center:
            {
                textJustify = new Vector2(0.5f, 0f);
                break;
            }
            case Alignment.Right:
            {
                textJustify = new Vector2(1f, 0f);
                break;
            }
        }
            
        resultText = MathF.Round(displayAmount, decimalPartLength).ToString() == null ? "0" : MathF.Round(displayAmount, decimalPartLength).ToString();

        
        switch(showTitle)
        {
            case ShowCounterTitleMod.Before:
            {
                resultText = countPerMinute ? toCustomTitle("IPM: ") + MathF.Round(displayAmount, decimalPartLength).ToString() :
                toCustomTitle("IPS: ") + MathF.Round(displayAmount, decimalPartLength).ToString();
                break;
            }
            case ShowCounterTitleMod.After:
            {
                resultText = countPerMinute ? MathF.Round(displayAmount, decimalPartLength).ToString() + 
                toCustomTitle(" IPM") : MathF.Round(displayAmount, decimalPartLength).ToString() + toCustomTitle(" IPS");
                break;
            }
        }



        if (outline)
        {
            Fonts.Get("Renogare").DrawEdgeOutline(64,
            resultText, CalculateActualPos(TextPos), textJustify,
            Vector2.One * (float)(size / 100f),
            new Color((float)(red / 255f), (float)(green / 255f), (float)(blue / 255f)),
            4f, (grayScale / 255f > 0.5f) ? Color.Black : Color.White, 1f,
            (grayScale / 255f > 0.5f) ? Color.Black : Color.White);
        }
        else
        {
            Fonts.Get("Renogare").Draw(64,
            resultText,
            CalculateActualPos(TextPos),
            textJustify,
            Vector2.One * (float)(size / 100f),
            new Color((float)(red / 255f) * (float)(opacity / 100f), (float)(green / 255f)
            * (float)(opacity / 100f), (float)(blue / 255f) * (float)(opacity / 100f), (float)(opacity / 100f)));
        }

        
        if (countPerMinute && method != CalcMethod.GapSaving)
            displayAmount /= 60f;


        if (showResultRoomCPM)
            DrawRrc();

        base.Render();
        return;
    }
    private static float textToLength(string input)
    {
        float result = 0f;
        for (int c = 0; c < input.Length; c++)
        {
            result += (float)(Fonts.Get("Renogare").Get(64f).Measure(input[c]).X *
                ((float)((float)Celeste.GameWidth / (float)Celeste.TargetWidth)))
                * (float)(size / 100f);
        }
        return result;
    }
    private static string toCustomTitle(string input)
    {
        return (APMmeterModule.Settings.TextCustomization.CustomCounterTitle != "" ?
                APMmeterModule.Settings.TextCustomization.CustomCounterTitle : input);
    }
    private static void DrawTextBG()
    {
        if (textToLength(resultText) >= peak.size)
            peak = new Gap(APMmeterModule.Settings.BackGroundSettings.PeakLifeTime, textToLength(resultText));

        if (peak.lifeTime <= 0f)
            peak.size -= (float)Engine.DeltaTime * (float)APMmeterModule.Settings.BackGroundSettings.PeakDecreaseSpeed;
        else
            peak.lifeTime -= Engine.DeltaTime;

        float textWidth = peak.size;


        bool leftWall = (TextPos.X < 4f * size / 100f) && (align == Alignment.Left);
        bool rightWall = (TextPos.X > 320f - (4f * size / 100f)) && (align == Alignment.Right);

        Vector2 rectPos = CalculateActualPos(TextPos + offset);

        if (leftWall)
        {
            Draw.Rect(CalculateActualPos(new Vector2(-1f, TextPos.Y + offset.Y)),
            toActualWidth(textWidth + TextPos.X + 1f),
            toActualHeight(4f * size / 100f),
            (grayScale / 255f > 0.5f) ? Color.Black : Color.White);

        }
        if(rightWall)
        {
            rectPos = CalculateActualPos(new Vector2(TextPos.X - textWidth, TextPos.Y + offset.Y));
            Draw.Rect(rectPos,
            toActualWidth(textWidth + 320f - TextPos.X + 1f),
            toActualHeight(4f * size / 100f),
            (grayScale / 255f > 0.5f) ? Color.Black : Color.White);
        }
        else
        {
            switch (align)
            { 
                case Alignment.Left:
                {
                    Draw.Rect(rectPos,
                    toActualWidth(textWidth),
                    toActualHeight(4f * size / 100f),
                    (grayScale / 255f > 0.5f) ? Color.Black : Color.White);
                    break;
                }
                case Alignment.Center:
                {
                    rectPos = CalculateActualPos(new Vector2(TextPos.X - textWidth / 2f, TextPos.Y + offset.Y));
                    Draw.Rect(rectPos,
                    toActualWidth(textWidth),
                    toActualHeight(4f * size / 100f),
                    (grayScale / 255f > 0.5f) ? Color.Black : Color.White);
                    break;
                }
                case Alignment.Right:
                {
                    rectPos = CalculateActualPos(new Vector2(TextPos.X - textWidth, TextPos.Y + offset.Y));
                    Draw.Rect(rectPos,
                    toActualWidth(textWidth),
                    toActualHeight(4f * size / 100f),
                    (grayScale / 255f > 0.5f) ? Color.Black : Color.White);
                    break;
                }
            }

        }
        if(!rightWall) //right triangle
        {
            background.Draw(rectPos + new Vector2(toActualWidth(textWidth - 0.3f), toActualHeight(-0.05f)),
            Vector2.Zero, (grayScale / 255f > 0.5f) ? Color.Black : Color.White, toActualHeight(4f * size / 100f) / 200f);
        }

        if (!leftWall) //left triangle
        {
            background.Draw(rectPos  + new Vector2(0f, toActualHeight(-0.05f)), new Vector2(1f, 0f),
            (grayScale / 255f > 0.5f) ? Color.Black : Color.White, new Vector2(toActualHeight(4f * size / 100f) / 200f * -1f,
            toActualHeight(4f * size / 100f) / 200f));
        }
    }

    private static void DrawRrc()
    {
        if (resultCPM.Length > 0)
        {
            for (int c = 0; c < resultCPM.Length; c++)
            {
                resultCPM[c].timer += Engine.DeltaTime;
                if (resultCPM[c].timer >= srrcDuration)
                {
                    if (resultCPM.Length == 1)
                    {
                         resultCPM = new ResultRoomCPM[0];
                    }
                    else
                    {
                        ResultRoomCPM[] bufferArr = new ResultRoomCPM[resultCPM.Length - 1];
                        bufferArr = resultCPM.Skip(c).ToArray();
                        resultCPM = new ResultRoomCPM[bufferArr.Length];
                        resultCPM = bufferArr;
                    }
                }
            }
        }

        if (resultCPM.Length <= 0)
            return;
        
        for (int c=0; c<resultCPM.Length; c++)
        {
            if (resultCPM[c].timer >= srrcDuration)
                resultCPM[c].timer = srrcDuration;

            resultCPM[c].pos = srrcPosCurve.GetPoint(Ease.SineInOut((float)(resultCPM[c].timer /srrcDuration)));
            string result = countPerMinute? MathF.Round(resultCPM[c].number*60f, decimalPartLength).ToString() :
                MathF.Round(resultCPM[c].number, decimalPartLength).ToString();
            if (result == "NaN")
                result = "0";
            switch (showTitle)
            {
                case ShowCounterTitleMod.Before:
                {
                    result = countPerMinute ? toCustomTitle("IPM: ") + result :
                    toCustomTitle("IPS: ") + result;
                    break;
                    }
                case ShowCounterTitleMod.After:
                {
                    result = countPerMinute ? result + toCustomTitle(" IPM") :
                    result + toCustomTitle(" IPS");
                    break;
                }
            }

            Fonts.Get("Renogare").Draw(64,
            result,
            CalculateActualPos(resultCPM[c].pos),
            textJustify,
            Vector2.One * (float)(size / 100f)*(srrcSizeCurve.GetPoint(Ease.SineInOut((float)(resultCPM[c].timer / srrcDuration))).Y),
            (APMmeterModule.Settings.ResultRoomValueSettings.CopyTextParams? new Color((float)(red / 255f), (float)(green / 255f), (float)(blue / 255f)) : Color.LightBlue) * srrcOpacityCurve.GetPoint(Ease.SineInOut((float)(resultCPM[c].timer / srrcDuration))).Y);

        }
    }

    private static Vector2 CalculateActualPos(Vector2 input)
    {
        return new Vector2(input.X * Celeste.TargetWidth / Celeste.GameWidth,
            input.Y * Celeste.TargetHeight / Celeste.GameHeight);
    }

    private static float toActualWidth(float input)
    {
        return (input * Celeste.TargetWidth / Celeste.GameWidth);
    }
    private static float toActualHeight(float input)
    {
        return (input * Celeste.TargetHeight / Celeste.GameHeight);
    }
}   